@@ -35,6 +35,7 @@
 #include <QPainter>
 #include <QScrollBar>
 #include <QPaintEvent>
+#include <algorithm>
 
 #include "kcategorydrawer.h"
 #include "kcategorizedsortfilterproxymodel.h"
@@ -308,7 +309,7 @@ void KCategorizedView::Private::rowsInserted(const QModelIndex &parent, int star
         const QModelIndex firstAffectedCategory = blocks[category].firstIndex;
         //BEGIN: order for marking as alternate those blocks that are alternate
         QList<Block> blockList = blocks.values();
-        qSort(blockList.begin(), blockList.end(), Block::lessThan);
+        std::sort(blockList.begin(), blockList.end(), Block::lessThan);
         QList<int> firstIndexesRows;
         foreach (const Block &block, blockList) {
             firstIndexesRows << block.firstIndex.row();
@@ -1317,7 +1318,7 @@ void KCategorizedView::rowsAboutToBeRemoved(const QModelIndex &parent,
     {
         //BEGIN: order for marking as alternate those blocks that are alternate
         QList<Private::Block> blockList = d->blocks.values();
-        qSort(blockList.begin(), blockList.end(), Private::Block::lessThan);
+        std::sort(blockList.begin(), blockList.end(), Private::Block::lessThan);
         QList<int> firstIndexesRows;
         foreach (const Private::Block &block, blockList) {
             firstIndexesRows << block.firstIndex.row();
