@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2020 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -92,7 +92,7 @@ static inline QString mpdLyricsFilePath(const Song &song)
 
 static inline QString fixNewLines(const QString &o)
 {
-    return QString(o).replace(QLatin1String("\n\n\n"), QLatin1String("\n\n")).replace("\n", "<br/>");
+    return QString(o).replace("\\n", "\n").replace("\\t", " ").replace("\t", " ").replace(QLatin1String("\n\n\n"), QLatin1String("\n\n")).replace("\n", "<br/>");
 }
 
 SongView::SongView(QWidget *p)
@@ -209,6 +209,16 @@ void SongView::search()
 
 void SongView::edit()
 {
+    if (lyricsFile.isEmpty()) {
+        lyricsFile=lyricsCacheFileName(currentSong, true);
+    }
+    if (!QFile::exists(lyricsFile)) {
+        QFile file(lyricsFile);
+        if (file.open(QIODevice::ReadWrite|QIODevice::Text)) {
+            QTextStream stream(&file);
+            stream << tr("Failed to locate lyrics") << endl;
+        }
+    }
     QDesktopServices::openUrl(QUrl::fromLocalFile(lyricsFile));
 }
 
@@ -464,9 +474,13 @@ static inline QString fixNewLine(QString s)
     return s.replace(QLatin1String("\n"), QLatin1String("<br/>"));
 }
 
-static QString createRow(const QString &key, const QString &value)
+static QString createRow(const QString &key, const QString &value, bool htmlEscape = true)
 {
-    return value.isEmpty() ? QString() : QString("<tr><td>%1:&nbsp;</td><td>%2</td></tr>").arg(key).arg(fixNewLine(value));
+    return value.isEmpty()
+            ? QString()
+            : htmlEscape
+                ? QString("<tr><td>%1:&nbsp;</td><td>%2</td></tr>").arg(key).arg(fixNewLine(value).toHtmlEscaped())
+                : QString("<tr><td>%1:&nbsp;</td><td>%2</td></tr>").arg(key).arg(fixNewLine(value));
 }
 
 struct MapEntry {
@@ -486,6 +500,13 @@ static QString clean(QString v)
 }
 #endif
 
+static void addIfNotEmpty(QMultiMap<int, QString> &tags, int pos, const QString &key, const QString &val)
+{
+    if (!val.isEmpty()) {
+        tags.insert(pos, createRow(key, val));
+    }
+}
+
 void SongView::loadMetadata()
 {
     if (!metadataNeedsUpdating) {
@@ -587,16 +608,18 @@ void SongView::loadMetadata()
 
     if (tags.isEmpty()) {
         int pos=0;
-        tags.insert(pos++, createRow(tr("Artist"), currentSong.artist));
-        tags.insert(pos++, createRow(tr("Album artist"), currentSong.albumartist));
-        tags.insert(pos++, createRow(tr("Composer"), currentSong.composer()));
-        tags.insert(pos++, createRow(tr("Performer"), currentSong.performer()));
-        tags.insert(pos++, createRow(tr("Album"), currentSong.album));
-        tags.insert(pos++, createRow(tr("Track number"), 0==currentSong.track ? QString() : QString::number(currentSong.track)));
-        tags.insert(pos++, createRow(tr("Disc number"), 0==currentSong.disc ? QString() : QString::number(currentSong.disc)));
-        tags.insert(pos++, createRow(tr("Genre"), currentSong.displayGenre()));
-        tags.insert(pos++, createRow(tr("Year"), 0==currentSong.track ? QString() : QString::number(currentSong.year)));
-        tags.insert(pos++, createRow(tr("Comment"), fixNewLine(currentSong.comment())));
+        addIfNotEmpty(tags, pos++, tr("Artist"), currentSong.artist);
+        addIfNotEmpty(tags, pos++, tr("Album artist"), currentSong.albumartist);
+        addIfNotEmpty(tags, pos++, tr("Composer"), currentSong.composer());
+        addIfNotEmpty(tags, pos++, tr("Performer"), currentSong.performer());
+        addIfNotEmpty(tags, pos++, tr("Album"), currentSong.album);
+        addIfNotEmpty(tags, pos++, tr("Track number"), 0==currentSong.track ? QString() : QString::number(currentSong.track));
+        addIfNotEmpty(tags, pos++, tr("Disc number"), 0==currentSong.disc ? QString() : QString::number(currentSong.disc));
+        addIfNotEmpty(tags, pos++, tr("Duration"), 0==currentSong.time ? QString() : Utils::formatTime(currentSong.time));
+        addIfNotEmpty(tags, pos++, tr("Genre"), currentSong.displayGenre());
+        addIfNotEmpty(tags, pos++, tr("Year"), 0==currentSong.track ? QString() : QString::number(currentSong.year));
+        addIfNotEmpty(tags, pos++, tr("Original Year"), 0==currentSong.origYear ? QString() : QString::number(currentSong.origYear));
+        addIfNotEmpty(tags, pos++, tr("Comment"), fixNewLine(currentSong.comment()));
     }
 
     QString tagInfo;
@@ -633,7 +656,7 @@ void SongView::loadMetadata()
     QString songFile=currentSong.filePath(MPDConnection::self()->getDetails().dir);
     if (QFile::exists(songFile)) {
         tagInfo+=createRow(tr("Filename"), QLatin1String("<a href=\"file://")+songFile+QLatin1String("\">")+
-                                           songFile+QLatin1String("</a>"));
+                                           songFile.toHtmlEscaped()+QLatin1String("</a>"), false);
     } else {
         tagInfo+=createRow(tr("Filename"), songFile);
     }
@@ -713,7 +736,9 @@ void SongView::abort()
 
         text->setText(QString());
         // Set lyrics file anyway - so that editing is enabled!
-        lyricsFile=lyricsCacheFileName(currentSong);
+        lyricsFile=Settings::self()->storeLyricsInMpdDir() && !currentSong.isNonMPD()
+                ? mpdLyricsFilePath(currentSong)
+                : lyricsCacheFileName(currentSong);
         setMode(Mode_Display);
     }
     cancelJobAction->setEnabled(false);
@@ -816,7 +841,10 @@ void SongView::lyricsReady(int id, QString lyrics)
         } else {
             text->setText(fixNewLines(plain));
             lyricsFile=QString();
-            saveFile(lyricsCacheFileName(currentSong, true));
+            if (! ( Settings::self()->storeLyricsInMpdDir() && !currentSong.isNonMPD() &&
+                    saveFile(mpdLyricsFilePath(currentSong))) ) {
+                saveFile(lyricsCacheFileName(currentSong, true));
+            }
             setMode(Mode_Display);
         }
     }
@@ -863,7 +891,9 @@ void SongView::getLyrics()
         text->setText(QString());
         currentProvider=-1;
         // Set lyrics file anyway - so that editing is enabled!
-        lyricsFile=lyricsCacheFileName(currentSong);
+        lyricsFile=Settings::self()->storeLyricsInMpdDir() && !currentSong.isNonMPD()
+                        ? mpdLyricsFilePath(currentSong)
+                        : lyricsCacheFileName(currentSong);
         setMode(Mode_Display);
     }
 }
