@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2020 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -153,9 +153,9 @@ void View::clear()
 void View::setHeader(const QString &str)
 {
     #ifdef CONTEXT_CENTERED
-    header->setText("<"+headerTag+" width=\"100%\" align=\"center\">"+str+"</"+headerTag+">");
+    header->setText("<"+headerTag+" width=\"100%\" align=\"center\">"+str.toHtmlEscaped()+"</"+headerTag+">");
     #else
-    header->setText("<"+headerTag+">"+str+"</"+headerTag+">");
+    header->setText("<"+headerTag+">"+str.toHtmlEscaped()+"</"+headerTag+">");
     #endif
 }
 
@@ -258,27 +258,6 @@ void View::addEventFilter(QObject *obj)
     header->installEventFilter(obj);
 }
 
-void View::setZoom(int z)
-{
-    for (TextBrowser *t: texts) {
-        t->setZoom(z);
-    }
-    QFont f=header->font();
-    f.setPointSize(f.pointSize()+z);
-    header->setFont(f);
-
-    if (selector) {
-        QFont f=selector->font();
-        f.setPointSize(f.pointSize()+z);
-        selector->setFont(f);
-    }
-}
-
-int View::getZoom()
-{
-    return text->zoom();
-}
-
 void View::setHtml(const QString &h, int index)
 {
     texts.at(index)->setText(QLatin1String("<html><head><style type=text/css>a:link {color:")+text->palette().link().color().name()+
