@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2017-2018 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2017-2020 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -29,6 +29,7 @@
 #include <QFile>
 #include <QRegExp>
 #include <QDebug>
+#include <algorithm>
 
 static const int constSchemaVersion=5;
 
@@ -188,7 +189,7 @@ static bool songSort(const Song &a, const Song &b)
         if (0!=cmp) {
             return cmp<0;
         }
-        a.artist.localeAwareCompare(b.artist);
+        cmp=a.artist.localeAwareCompare(b.artist);
         if (0!=cmp) {
             return cmp<0;
         }
@@ -649,7 +650,7 @@ void LibraryDb::insertSong(const Song &s)
     QString albumId=s.albumId();
     insertSongQuery->bindValue(":file", s.file);
     insertSongQuery->bindValue(":artist", s.artist);
-    insertSongQuery->bindValue(":artistId", s.artistOrComposer());
+    insertSongQuery->bindValue(":artistId", s.albumArtistOrComposer());
     insertSongQuery->bindValue(":albumArtist", s.albumartist);
     insertSongQuery->bindValue(":artistSort", artistSort(s));
     insertSongQuery->bindValue(":composer", s.composer());
@@ -704,7 +705,7 @@ QList<LibraryDb::Genre> LibraryDb::getGenres()
         DBUG << it.key();
         genres.append(Genre(it.key(), it.value().size()));
     }
-    qSort(genres);
+    std::sort(genres.begin(), genres.end());
     return genres;
 }
 
@@ -737,7 +738,7 @@ QList<LibraryDb::Artist> LibraryDb::getArtists(const QString &genre)
 //        DBUG << it.key();
         artists.append(Artist(it.key(), sortMap[it.key()], it.value()));
     }
-    qSort(artists);
+    std::sort(artists.begin(), artists.end());
     return artists;
 }
 
@@ -852,25 +853,25 @@ QList<LibraryDb::Album> LibraryDb::getAlbums(const QString &artistId, const QStr
     DBUG << "After select" << timer.elapsed();
     switch(sort) {
     case AS_AlArYr:
-        qSort(albums.begin(), albums.end(), albumsSortAlArYr);
+        std::sort(albums.begin(), albums.end(), albumsSortAlArYr);
         break;
     case AS_AlYrAr:
-        qSort(albums.begin(), albums.end(), albumsSortAlYrAr);
+        std::sort(albums.begin(), albums.end(), albumsSortAlYrAr);
         break;
     case AS_ArAlYr:
-        qSort(albums.begin(), albums.end(), albumsSortArAlYr);
+        std::sort(albums.begin(), albums.end(), albumsSortArAlYr);
         break;
     case AS_ArYrAl:
-        qSort(albums.begin(), albums.end(), albumsSortArYrAl);
+        std::sort(albums.begin(), albums.end(), albumsSortArYrAl);
         break;
     case AS_YrAlAr:
-        qSort(albums.begin(), albums.end(), albumsSortYrAlAr);
+        std::sort(albums.begin(), albums.end(), albumsSortYrAlAr);
         break;
     case AS_YrArAl:
-        qSort(albums.begin(), albums.end(), albumsSortYrArAl);
+        std::sort(albums.begin(), albums.end(), albumsSortYrArAl);
         break;
     case AS_Modified:
-        qSort(albums.begin(), albums.end(), albumsSortModified);
+        std::sort(albums.begin(), albums.end(), albumsSortModified);
         break;
     default:
         break;
@@ -908,19 +909,19 @@ QList<Song> LibraryDb::getTracks(const QString &artistId, const QString &albumId
 
     switch(sort) {
     case AS_AlArYr:
-        qSort(songs.begin(), songs.end(), songsSortAlAr);
+        std::sort(songs.begin(), songs.end(), songsSortAlAr);
         break;
     case AS_ArAlYr:
-        qSort(songs.begin(), songs.end(), songsSortArAl);
+        std::sort(songs.begin(), songs.end(), songsSortArAl);
         break;
 //    case AS_Year:
-//        qSort(songs.begin(), songs.end(), songsSortYrAlAr);
+//        std::sort(songs.begin(), songs.end(), songsSortYrAlAr);
 //        break;
 //    case AS_Modified:
-//        qSort(songs.begin(), songs.end(), songsSortModified);
+//        std::sort(songs.begin(), songs.end(), songsSortModified);
 //        break;
     default:
-        qSort(songs.begin(), songs.end(), songSort);
+        std::sort(songs.begin(), songs.end(), songSort);
         break;
     }
     return songs;
@@ -978,7 +979,7 @@ QList<Song> LibraryDb::songs(const QStringList &files, bool allowPlaylists) cons
     return songList;
 }
 
-QList<LibraryDb::Album> LibraryDb::getAlbumsWithArtist(const QString &artist)
+QList<LibraryDb::Album> LibraryDb::getAlbumsWithArtistOrComposer(const QString &artist)
 {
     QList<LibraryDb::Album> albums;
     if (0!=currentVersion && db) {
@@ -993,7 +994,20 @@ QList<LibraryDb::Album> LibraryDb::getAlbumsWithArtist(const QString &artist)
         }
     }
 
-    qSort(albums.begin(), albums.end(), albumsSortArAlYr);
+    if (albums.isEmpty()) {
+        // No artist albums? Try composer...
+        SqlQuery query("distinct album, albumId, albumSort", *db);
+        query.addWhere("composer", artist);
+        query.exec();
+        DBUG << query.executedQuery();
+        while (query.next()) {
+            QString album=query.value(0).toString();
+            QString albumId=query.value(1).toString();
+            albums.append(Album(album.isEmpty() ? albumId : album, albumId, query.value(2).toString(), artist));
+        }
+    }
+
+    std::sort(albums.begin(), albums.end(), albumsSortArAlYr);
 
     return albums;
 }
@@ -1122,7 +1136,7 @@ bool LibraryDb::songExists(const Song &song)
         return false;
     }
     SqlQuery query("file", *db);
-    query.addWhere("artistId", song.artistOrComposer());
+    query.addWhere("artistId", song.albumArtistOrComposer());
     query.addWhere("albumId", song.albumId());
     query.addWhere("title", song.title);
     query.addWhere("track", song.track);
