@@ -25,37 +25,37 @@
 #define _SYNCCOLLECTIONWIDGET_H_
 
 #include "ui_synccollectionwidget.h"
-#include "mpd/song.h"
+#include "mpd-interface/song.h"
 #include "models/musiclibrarymodel.h"
+#include "models/musiclibraryproxymodel.h"
 #include <QSet>
 
 class QTimer;
-class MusicLibraryProxyModel;
 class Action;
-class Icon;
 
 class SyncCollectionWidget : public QWidget, Ui::SyncCollectionWidget
 {
     Q_OBJECT
 
 public:
-    SyncCollectionWidget(QWidget *parent, const QString &title, const QString &action, bool showCovers=true);
-    virtual ~SyncCollectionWidget();
+    SyncCollectionWidget(QWidget *parent, const QString &title);
+    ~SyncCollectionWidget() override;
 
-    void update(const QSet<Song> &songs) { model->update(songs); }
-    void setSupportsAlbumArtistTag(bool s) { model->setSupportsAlbumArtistTag(s); }
-    int numArtists() { return model->rowCount(); }
-    void setIcon(const Icon &icon);
+    void clear() { model.clear(); }
+    void update(const QSet<Song> &songs);
+    void setSupportsAlbumArtistTag(bool s) { model.setSupportsAlbumArtistTag(s); }
+    int numArtists() { return model.rowCount(); }
+    int numCheckedSongs() const { return checked.count(); }
+    QList<Song> checkedSongs() const;
 
 Q_SIGNALS:
-    void copy(const QList<Song> &songs);
+    void selectionChanged();
+    void configure();
 
 private Q_SLOTS:
-    void songsChecked(const QSet<Song> &songs);
     void dataChanged(const QModelIndex &tl, const QModelIndex &br);
     void checkItems();
     void unCheckItems();
-    void copySongs();
     void delaySearchItems();
     void searchItems();
     void expandAll();
@@ -65,15 +65,15 @@ private Q_SLOTS:
 
 private:
     void checkItems(bool c);
-    void updateStats();
+    void songToggled(const MusicLibraryItemSong *song);
 
 private:
     bool performedSearch;
-    MusicLibraryModel *model;
-    MusicLibraryProxyModel *proxy;
+    MusicLibraryModel model;
+    MusicLibraryProxyModel proxy;
     QTimer *searchTimer;
-    QSet<Song> checkedSongs;
-    Action *copyAction;
+    QSet<const Song *> checked;
+    quint64 spaceRequired;
     Action *checkAction;
     Action *unCheckAction;
 };
