@@ -24,36 +24,53 @@
 #ifndef _SYNC_DIALOG_H_
 #define _SYNC_DIALOG_H_
 
-#include "dialog.h"
-#include "song.h"
+#include "support/dialog.h"
+#include "mpd-interface/song.h"
+#include "deviceoptions.h"
 
 class Device;
 class SyncCollectionWidget;
+class SqueezedTextLabel;
 
 class SyncDialog : public Dialog
 {
     Q_OBJECT
 
+    enum State {
+        State_Lists,
+        State_CopyToDevice,
+        State_CopyToLib
+    };
+
 public:
     static int instanceCount();
 
     SyncDialog(QWidget *parent);
-    virtual ~SyncDialog();
+    ~SyncDialog() override;
 
     void sync(const QString &udi);
 
 private Q_SLOTS:
     void copy(const QList<Song> &songs);
-    bool updateSongs(bool firstRun=false);
+    void librarySongs(const QList<Song> &songs, double pc);
+    void selectionChanged();
+    void configure();
+    void saveProperties(const QString &path, const DeviceOptions &opts);
 
 private:
+    void updateSongs();
+    void slotButtonClicked(int button) override;
     Device * getDevice();
 
 private:
+    State state;
+    SqueezedTextLabel *statusLabel;
     QString devUdi;
     Device *currentDev;
     SyncCollectionWidget *devWidget;
     SyncCollectionWidget *libWidget;
+    QSet<Song> libSongs;
+    DeviceOptions libOptions;
 };
 
 #endif
