@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2020 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -23,25 +23,21 @@
 
 #include "umsdevice.h"
 #include "support/utils.h"
+#include "support/monoicon.h"
 #include "devicepropertiesdialog.h"
 #include "devicepropertieswidget.h"
 #include "actiondialog.h"
-#include "support/localize.h"
 #include <QDir>
 #include <QFile>
 #include <QFileInfo>
 #include <QTextStream>
-#ifdef ENABLE_KDE_SUPPORT
-#include <solid/storagedrive.h>
-#else
 #include "solid-lite/storagedrive.h"
-#endif
 
 static const QLatin1String constSettingsFile("/.is_audio_player");
 static const QLatin1String constMusicFolderKey("audio_folder");
 static const QLatin1String constCollectionNameKey("collection_name");
 
-UmsDevice::UmsDevice(MusicModel *m, Solid::Device &dev)
+UmsDevice::UmsDevice(MusicLibraryModel *m, Solid::Device &dev)
     : FsDevice(m, dev)
     , access(dev.as<Solid::StorageAccess>())
 {
@@ -61,6 +57,7 @@ UmsDevice::UmsDevice(MusicModel *m, Solid::Device &dev)
     defaultName=data()+details;
     setData(defaultName);
     setup();
+    icn=MonoIcon::icon(FontAwesome::usb, Utils::monoIconColor());
 }
 
 UmsDevice::~UmsDevice()
@@ -75,7 +72,7 @@ void UmsDevice::connectionStateChanged()
         if (opts.autoScan || scanned){ // Only scan if we are set to auto scan, or we have already scanned before...
             rescan(false); // Read from cache if we have it!
         } else {
-            setStatusMessage(i18n("Not Scanned"));
+            setStatusMessage(tr("Not Scanned"));
         }
     } else {
         clear();
@@ -117,10 +114,10 @@ QString UmsDevice::capacityString()
         return statusMessage();
     }
     if (!isConnected()) {
-        return i18n("Not Connected");
+        return tr("Not Connected");
     }
 
-    return i18n("%1 free", Utils::formatByteSize(spaceInfo.size()-spaceInfo.used()));
+    return tr("%1 free").arg(Utils::formatByteSize(spaceInfo.size()-spaceInfo.used()));
 }
 
 qint64 UmsDevice::freeSpace()
@@ -192,7 +189,7 @@ void UmsDevice::setup()
         dirs << QLatin1String("Music") << QLatin1String("MUSIC")
              << QLatin1String("Albums") << QLatin1String("ALBUMS");
 
-        foreach (const QString &d, dirs) {
+        for (const QString &d: dirs) {
             if (QDir(path+d).exists()) {
                 audioFolder=path+d;
                 break;
@@ -207,7 +204,7 @@ void UmsDevice::setup()
     if (opts.autoScan || scanned){ // Only scan if we are set to auto scan, or we have already scanned before...
         rescan(false); // Read from cache if we have it!
     } else {
-        setStatusMessage(i18n("Not Scanned"));
+        setStatusMessage(tr("Not Scanned"));
     }
     if (!opts.name.isEmpty() && opts.name!=n) {
         setData(opts.name);
@@ -283,7 +280,7 @@ void UmsDevice::saveOptions()
             out << constCollectionNameKey << '=' << opts.name << '\n';
         }
 
-        foreach (const QString &u, unusedParams) {
+        for (const QString &u: unusedParams) {
             out << u << '\n';
         }
     }
@@ -330,3 +327,5 @@ void UmsDevice::saveProperties(const QString &newPath, const DeviceOptions &newO
         emit renamed();
     }
 }
+
+#include "moc_umsdevice.cpp"
