@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2020 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -57,6 +57,7 @@
 #include <QApplication>
 #include <QJsonParseError>
 #include <QJsonDocument>
+#include <QDesktopWidget>
 
 GLOBAL_STATIC(Covers, instance)
 
@@ -123,7 +124,8 @@ static QImage scale(const Song &song, const QImage &img, int size)
 static bool canSaveTo(const QString &dir)
 {
     QString mpdDir=MPDConnection::self()->getDetails().dir;
-    return !dir.isEmpty() && !mpdDir.isEmpty() && !mpdDir.startsWith(QLatin1String("http://")) && QDir(mpdDir).exists() && dir.startsWith(mpdDir);
+    return !dir.isEmpty() && !mpdDir.isEmpty() && !mpdDir.startsWith(QLatin1String("http://"), Qt::CaseInsensitive) &&
+           !mpdDir.startsWith(QLatin1String("https://"), Qt::CaseInsensitive)  && QDir(mpdDir).exists() && dir.startsWith(mpdDir);
 }
 
 static const QString typeFromRaw(const QByteArray &raw)
@@ -640,7 +642,9 @@ void CoverDownloader::download(const Song &song)
 
     if (!song.isArtistImageRequest() && !song.isComposerImageRequest() && MPDConnection::self()->supportsCoverDownload()) {
         downloadViaMpd(job);
-    } else if (!MPDConnection::self()->getDetails().dir.isEmpty() && MPDConnection::self()->getDetails().dir.startsWith(QLatin1String("http://"))) {
+    } else if (!MPDConnection::self()->getDetails().dir.isEmpty() &&
+               (MPDConnection::self()->getDetails().dir.startsWith(QLatin1String("http://"), Qt::CaseInsensitive) ||
+                MPDConnection::self()->getDetails().dir.startsWith(QLatin1String("https://"), Qt::CaseInsensitive))) {
         downloadViaHttp(job, JobHttpJpg);
     } else if (fetchCovers || job.song.isArtistImageRequest()) {
         downloadViaRemote(job);
@@ -696,14 +700,17 @@ void CoverDownloader::downloadViaRemote(Job &job)
         url=QUrl("https://ws.audioscrobbler.com/2.0/");
         QUrlQuery query;
 
-        query.addQueryItem("method", job.song.isArtistImageRequest() || job.song.isComposerImageRequest() ? "artist.getInfo" : "album.getInfo");
+        query.addQueryItem("method", "artist.getinfo");
         ApiKeys::self()->addKey(query, ApiKeys::LastFm);
         query.addQueryItem("autocorrect", "1");
         query.addQueryItem("artist", Covers::fixArtist(job.song.albumArtist()));
-        if (!job.song.isArtistImageRequest()) {
-            query.addQueryItem("album", job.song.album);
-        }
         url.setQuery(query);
+
+        NetworkJob *j = network()->get(url);
+        connect(j, SIGNAL(finished()), this, SLOT(lastFmArtistCallFinished()));
+        job.type=JobRemote;
+        jobs.insert(j, job);
+        DBUG << url.toString();
     } else {
         url=QUrl("http://itunes.apple.com/search");
         QUrlQuery query;
@@ -712,13 +719,12 @@ void CoverDownloader::downloadViaRemote(Job &job)
         query.addQueryItem("media", "music");
         query.addQueryItem("entity", "album");
         url.setQuery(query);
+        NetworkJob *j = network()->get(url);
+        connect(j, SIGNAL(finished()), this, SLOT(remoteCallFinished()));
+        job.type=JobRemote;
+        jobs.insert(j, job);
+        DBUG << url.toString();
     }
-
-    NetworkJob *j = network()->get(url);
-    connect(j, SIGNAL(finished()), this, SLOT(remoteCallFinished()));
-    job.type=JobRemote;
-    jobs.insert(j, job);
-    DBUG << url.toString();
 }
 
 void CoverDownloader::mpdAlbumArt(const Song &song, const QByteArray &data)
@@ -774,36 +780,29 @@ void CoverDownloader::remoteCallFinished()
 
         if (reply->ok()) {
             if (job.song.isArtistImageRequest()) {
-                QXmlStreamReader doc(reply->readAll());
-                QString largeUrl;
-                bool inSection=false;
-                bool isArtistImage=job.song.isArtistImageRequest();
+                QJsonParseError jsonParseError;
+                QVariantMap parsed=QJsonDocument::fromJson(reply->readAll(), &jsonParseError).toVariant().toMap();
+                bool ok=QJsonParseError::NoError==jsonParseError.error;
 
-                doc.setNamespaceProcessing(false);
-                while (!doc.atEnd()) {
-                    doc.readNext();
+                if (ok && !parsed.isEmpty()) {
+                    QVariantList artistbackgrounds;
 
-                    if (doc.isStartElement()) {
-                        if (!inSection && QLatin1String(isArtistImage ? "artist" : "album")==doc.name()) {
-                            inSection=true;
-                        } else if (inSection && QLatin1String("image")==doc.name()) {
-                            QString size=doc.attributes().value("size").toString();
-                            if (QLatin1String("extralarge")==size) {
-                                url = doc.readElementText();
-                            } else if (QLatin1String("large")==size) {
-                                largeUrl = doc.readElementText();
-                            }
-                            if (!url.isEmpty() && !largeUrl.isEmpty()) {
-                                break;
-                            }
+                    if (parsed.contains("artistthumb")) {
+                        artistbackgrounds=parsed["artistthumb"].toList();
+                    } else {
+                        QVariantMap artist=parsed[parsed.keys().first()].toMap();
+
+                        if (artist.contains("artistthumb")) {
+                            artistbackgrounds=artist["artistthumb"].toList();
                         }
-                    } else if (doc.isEndElement() && inSection && QLatin1String(isArtistImage ? "artist" : "album")==doc.name()) {
-                        break;
                     }
-                }
 
-                if (url.isEmpty() && !largeUrl.isEmpty()) {
-                    url=largeUrl;
+                    if (!artistbackgrounds.isEmpty()) {
+                        QVariantMap artistbackground=artistbackgrounds.first().toMap();
+                        if (artistbackground.contains("url")) {
+                            url=artistbackground["url"].toString();
+                        }
+                    }
                 }
             } else {
                 QJsonParseError jsonParseError;
@@ -836,6 +835,66 @@ void CoverDownloader::remoteCallFinished()
     reply->deleteLater();
 }
 
+void CoverDownloader::lastFmArtistCallFinished()
+{
+    NetworkJob *reply=qobject_cast<NetworkJob *>(sender());
+    if (!reply) {
+        return;
+    }
+
+    DBUG << "status" << reply->error() << reply->errorString();
+
+    QHash<NetworkJob *, Job>::Iterator it(jobs.find(reply));
+    QHash<NetworkJob *, Job>::Iterator end(jobs.end());
+
+    if (it!=end) {
+        Job job=it.value();
+        jobs.erase(it);
+        QUrl url;
+
+        if (reply->ok()) {
+            bool inSection=false;
+            QXmlStreamReader doc(reply->readAll());
+            QStringList musicBrainzIds;
+
+            doc.setNamespaceProcessing(false);
+            while (!doc.atEnd()) {
+                doc.readNext();
+
+                if (doc.isStartElement()) {
+                    if (!inSection && QLatin1String("artist")==doc.name()) {
+                        inSection=true;
+                    } else if (inSection && QLatin1String("mbid")==doc.name()) {
+                        QString id=doc.readElementText();
+                        if (id.length()>4) {
+                            musicBrainzIds.append(id);
+                        }
+                    }
+                } else if (doc.isEndElement() && inSection && QLatin1String("artist")==doc.name()) {
+                    inSection=false;
+                }
+            }
+
+            if (!musicBrainzIds.isEmpty()) {
+                url=QUrl("http://webservice.fanart.tv/v3/music/"+musicBrainzIds.first());
+                QUrlQuery query;
+                ApiKeys::self()->addKey(query, ApiKeys::FanArt);
+                url.setQuery(query);
+            }
+        }
+
+        if (url.isValid()) {
+            NetworkJob *j=network()->get(QNetworkRequest(url));
+            connect(j, SIGNAL(finished()), this, SLOT(remoteCallFinished()));
+            DBUG << "download" << url;
+            jobs.insert(j, job);
+        } else {
+            failed(job);
+        }
+    }
+    reply->deleteLater();
+}
+
 void CoverDownloader::jobFinished()
 {
     NetworkJob *reply=qobject_cast<NetworkJob *>(sender());
@@ -1189,7 +1248,16 @@ Covers::Covers()
     , loader(nullptr)
 {
     devicePixelRatio=qApp->devicePixelRatio();
-    cache.setMaxCost(15*1024*1024*devicePixelRatio);
+
+    // Use screen size to calculate max cost - Issue #1498
+    int maxCost = 0;
+    QDesktopWidget *dw=QApplication::desktop();
+    if (dw) {
+        QWidget w;
+        QSize sz = dw->availableGeometry(&w).size();
+        maxCost = sz.width() * sz.height() * 5; // *5 as 32-bit pixmap (so 4 bytes), + some wiggle rooom :-)
+    }
+    cache.setMaxCost(qMax(static_cast<int>(15*1024*1024*devicePixelRatio), maxCost)); // Ensure at least 15M
 }
 
 void Covers::readConfig()
@@ -1330,10 +1398,10 @@ QPixmap * Covers::get(const Song &song, int size, bool urgent)
         pix=cache.object(key);
 
         if (!pix) {
-            if (song.isArtistImageRequest() && song.isVariousArtists()) {
+            /*if (song.isArtistImageRequest() && song.isVariousArtists()) {
                 // Load artist image...
                 pix=new QPixmap(Icons::self()->artistIcon.pixmap(size, size).scaled(QSize(size, size), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
-            } else if (Song::SingleTracks==song.type) {
+            } else*/ if (Song::SingleTracks==song.type) {
                 pix=new QPixmap(Icons::self()->albumIcon(size).pixmap(size, size).scaled(QSize(size, size), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
             } else if (song.isStandardStream()) {
                 pix=new QPixmap(Icons::self()->streamIcon.pixmap(size, size).scaled(QSize(size, size), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
@@ -1666,8 +1734,9 @@ Covers::Image Covers::locateImage(const Song &song)
         }
     }
 
-    if (!songFile.isEmpty() && !songFile.startsWith("http:/") && !song.isCdda() &&
-        (haveAbsPath || song.isCantataStream() || (!MPDConnection::self()->getDetails().dir.isEmpty() && !MPDConnection::self()->getDetails().dir.startsWith(QLatin1String("http://")) ) ) ) {
+    if (!songFile.isEmpty() && !songFile.startsWith("http:/", Qt::CaseInsensitive) && !songFile.startsWith("https:/", Qt::CaseInsensitive) && !song.isCdda() &&
+        (haveAbsPath || song.isCantataStream() || (!MPDConnection::self()->getDetails().dir.isEmpty() && !MPDConnection::self()->getDetails().dir.startsWith(QLatin1String("http://"), Qt::CaseInsensitive)  &&
+                                                   !MPDConnection::self()->getDetails().dir.startsWith(QLatin1String("https://"), Qt::CaseInsensitive)) ) ) {
         dirName=song.isCantataStream() ? Utils::getDir(songFile)
                                        : songFile.endsWith(Utils::constDirSep)
                                             ? (haveAbsPath ? QString() : MPDConnection::self()->getDetails().dir)+songFile
@@ -1695,7 +1764,7 @@ Covers::Image Covers::locateImage(const Song &song)
             if (!song.isComposerImageRequest() && (song.isVariousArtists() || song.isNonMPD())) {
                 QString basicArtist=song.basicArtist();
                 dirName=MPDConnection::self()->getDetails().dirReadable ? MPDConnection::self()->getDetails().dir : QString();
-                if (!dirName.isEmpty() && !dirName.startsWith(QLatin1String("http:/"))) {
+                if (!dirName.isEmpty() && !dirName.startsWith(QLatin1String("http:/"), Qt::CaseInsensitive) && !dirName.startsWith(QLatin1String("https:/"), Qt::CaseInsensitive)) {
                     dirName+=basicArtist+Utils::constDirSep;
                     for (const QString &fileName: coverFileNames) {
                         DBUG_CLASS("Covers") << "Checking file" << QString(dirName+fileName);
@@ -1935,7 +2004,7 @@ void Covers::gotAlbumCover(const Song &song, const QImage &img, const QString &f
 {
     QString key=albumKey(song);
     currentImageRequests.remove(key);
-//    if (!img.isNull() && !fileName.isEmpty() && !fileName.startsWith("http:/")) {
+//    if (!img.isNull() && !fileName.isEmpty() && !fileName.startsWith("http:/", Qt::CaseInsensitive) && !fileName.startsWith("https:/", Qt::CaseInsensitive)  ) {
         mutex.lock();
         filenames.insert(key, fileName.isEmpty() ? constNoCover : fileName);
         mutex.unlock();
@@ -1956,7 +2025,7 @@ void Covers::gotArtistImage(const Song &song, const QImage &img, const QString &
 {
     QString key=artistKey(song);
     currentImageRequests.remove(key);
-//    if (!img.isNull() && !fileName.isEmpty() && !fileName.startsWith("http:/")) {
+//    if (!img.isNull() && !fileName.isEmpty() && !fileName.startsWith("http:/", Qt::CaseInsensitive) && !fileName.startsWith("https:/", Qt::CaseInsensitive)) {
         mutex.lock();
         filenames.insert(key, fileName.isEmpty() ? constNoCover : fileName);
         mutex.unlock();
@@ -1976,7 +2045,7 @@ void Covers::gotComposerImage(const Song &song, const QImage &img, const QString
 {
     QString key=composerKey(song);
     currentImageRequests.remove(key);
-//    if (!img.isNull() && !fileName.isEmpty() && !fileName.startsWith("http:/")) {
+//    if (!img.isNull() && !fileName.isEmpty() && !fileName.startsWith("http:/", Qt::CaseInsensitive) && !fileName.startsWith("https:/", Qt::CaseInsensitive)) {
         mutex.lock();
         filenames.insert(key, fileName.isEmpty() ? constNoCover : fileName);
         mutex.unlock();
