@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2020 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -43,6 +43,7 @@
 #include <QSystemTrayIcon>
 #include <QSysInfo>
 #include <QStyleFactory>
+#include <algorithm>
 
 #define REMOVE(w) \
     w->setVisible(false); \
@@ -177,7 +178,7 @@ InterfaceSettings::InterfaceSettings(QWidget *p)
     playQueueBackground_custom->setProperty(constValueProperty, PlayQueueView::BI_Custom);
     playQueueBackgroundFile->setDirMode(false);
     playQueueBackgroundFile->setFilter(tr("Images (*.png *.jpg)"));
-    int labelWidth=qMax(fontMetrics().width(QLatin1String("100%")), fontMetrics().width(tr("10px", "pixels")));
+    int labelWidth=qMax(fontMetrics().horizontalAdvance(QLatin1String("100%")), fontMetrics().horizontalAdvance(tr("10px", "pixels")));
     playQueueBackgroundOpacityLabel->setFixedWidth(labelWidth);
     playQueueBackgroundBlurLabel->setFixedWidth(labelWidth);
     connect(playQueueBackgroundOpacity, SIGNAL(valueChanged(int)), SLOT(setPlayQueueBackgroundOpacityLabel()));
@@ -218,10 +219,10 @@ InterfaceSettings::InterfaceSettings(QWidget *p)
     #ifdef ENABLE_DEVICES_SUPPORT
     coverNameNoteLabel->setText(tr("If no setting is specified for 'Filename', then Cantata will use a default of "
                                    "<code>cover</code>. This filename is used when downloading covers, or when adding "
-                                   "music to your library from devices."));
+                                   "music to your library from devices. This should be specified <b>without</b> an extension."));
     #else
     coverNameNoteLabel->setText(tr("If no setting is specified for 'Filename', then Cantata will use a default of "
-                                   "<code>cover</code>. This filename is used when downloading covers."));
+                                   "<code>cover</code>. This filename is used when downloading covers. This should be specified <b>without</b> an extension."));
     #endif
 
     QFont f = Utils::smallFont(ignorePrefixesLabel->font());
@@ -438,7 +439,7 @@ void InterfaceSettings::showEvent(QShowEvent *e)
 
         QString current = Settings::self()->lang();
         QStringList names = langMap.keys();
-        qStableSort(names.begin(), names.end(), localeAwareCompare);
+        std::stable_sort(names.begin(), names.end(), localeAwareCompare);
         lang->addItem(tr("System default"), QString());
         lang->setCurrentIndex(0);
         for (const QString &name: names) {
