@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2020 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -25,26 +25,14 @@
 #define PREFERENCES_DIALOG_H
 
 #include "config.h"
-#include "support/dialog.h"
-#include "support/pagewidget.h"
+#include "support/configdialog.h"
 
-#ifndef ENABLE_KDE_SUPPORT
 class ProxySettings;
 class ShortcutsSettingsPage;
-#endif
-
 class ServerSettings;
 class PlaybackSettings;
-class FileSettings;
 class InterfaceSettings;
-#ifdef ENABLE_STREAMS
-class StreamsSettings;
-#endif
-#ifdef ENABLE_ONLINE_SERVICES
-class OnlineSettings;
-#endif
 class ContextSettings;
-class HttpServerSettings;
 struct MPDConnectionDetails;
 class CacheSettings;
 #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
@@ -55,8 +43,10 @@ class QStringList;
 class ProxySettings;
 #endif
 class ScrobblingSettings;
+class CustomActionsSettings;
+class ApiKeysSettings;
 
-class PreferencesDialog : public Dialog
+class PreferencesDialog : public ConfigDialog
 {
     Q_OBJECT
 
@@ -64,10 +54,11 @@ public:
     static int instanceCount();
 
     PreferencesDialog(QWidget *parent);
-    virtual ~PreferencesDialog();
+    ~PreferencesDialog() override;
 
 private:
-    void slotButtonClicked(int button);
+    void save() override;
+    void cancel() override;
 
 public Q_SLOTS:
     void showPage(const QString &page);
@@ -79,33 +70,21 @@ Q_SIGNALS:
     void settingsSaved();
 
 private:
-    PageWidget *pageWidget;
     ServerSettings *server;
     PlaybackSettings *playback;
-    FileSettings *files;
     InterfaceSettings *interface;
-    #ifdef ENABLE_STREAMS
-    StreamsSettings *streams;
-    #endif
-    #ifdef ENABLE_ONLINE_SERVICES
-    OnlineSettings *online;
-    #endif
     ContextSettings *context;
-    #ifdef ENABLE_HTTP_SERVER
-    HttpServerSettings *http;
-    #endif
     #ifdef ENABLE_PROXY_CONFIG
     ProxySettings *proxy;
     #endif
-    #ifndef ENABLE_KDE_SUPPORT
     ShortcutsSettingsPage *shortcuts;
-    #endif
     CacheSettings *cache;
     #if defined CDDB_FOUND || defined MUSICBRAINZ5_FOUND
     AudioCdSettings *audiocd;
     #endif
     ScrobblingSettings *scrobbling;
-    QMap<QString, PageWidgetItem *> pages;
+    CustomActionsSettings *custom;
+    ApiKeysSettings *apiKeys;
 };
 
 #endif
