@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2020 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -170,6 +170,7 @@ MPDConnectionDetails Settings::connectionDetails(const QString &name)
         details.streamUrl=grp.get("streamUrl", QString());
         #endif
         details.replayGain=grp.get("replayGain", QString());
+        details.applyReplayGain=grp.get("applyReplayGain", true);
         // if the setting hasn't been set before, we set it to true to allow
         // for easy migration of existing settings
         details.allowLocalStreaming=grp.get("allowLocalStreaming", true);
@@ -182,6 +183,7 @@ MPDConnectionDetails Settings::connectionDetails(const QString &name)
         #ifdef ENABLE_HTTP_STREAM_PLAYBACK
         details.streamUrl=QString();
         #endif
+        details.applyReplayGain=true;
         details.allowLocalStreaming=true;
         details.autoUpdate=false;
     }
@@ -274,6 +276,11 @@ bool Settings::storeCoversInMpdDir()
     return cfg.get("storeCoversInMpdDir", false);
 }
 
+bool Settings::storeLyricsInMpdDir()
+{
+    return cfg.get("storeLyricsInMpdDir", false);
+}
+
 QString Settings::coverFilename()
 {
     QString name=cfg.get("coverFilename", QString());
@@ -718,15 +725,19 @@ void Settings::saveConnectionDetails(const MPDConnectionDetails &v)
     }
 
     QString n=MPDConnectionDetails::configGroupName(v.name);
-
     Configuration grp(n);
     grp.set("host", v.hostname);
     grp.set("port", (int)v.port);
-    grp.setDirPath("dir", v.dir);
+    if (v.dir.startsWith("http:", Qt::CaseInsensitive) || v.dir.startsWith("https:", Qt::CaseInsensitive)) {
+        grp.set("dir", v.dir);
+    } else {
+        grp.setDirPath("dir", v.dir);
+    }
     grp.set("passwd", v.password);
     #ifdef ENABLE_HTTP_STREAM_PLAYBACK
     grp.set("streamUrl", v.streamUrl);
     #endif
+    grp.set("applyReplayGain", v.applyReplayGain);
     grp.set("allowLocalStreaming", v.allowLocalStreaming);
     grp.set("autoUpdate", v.autoUpdate);
 }
@@ -803,6 +814,11 @@ void Settings::saveStoreCoversInMpdDir(bool v)
     cfg.set("storeCoversInMpdDir", v);
 }
 
+void Settings::saveStoreLyricsInMpdDir(bool v)
+{
+    cfg.set("storeLyricsInMpdDir", v);
+}
+
 void Settings::saveCoverFilename(const QString &v)
 {
     cfg.set("coverFilename", v);
