@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2020 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -30,7 +30,7 @@
 #include <QList>
 #include <QVariant>
 #include "musiclibraryitem.h"
-#include "mpd/song.h"
+#include "mpd-interface/song.h"
 
 class MusicLibraryItemAlbum;
 
@@ -38,33 +38,25 @@ class MusicLibraryItemSong : public MusicLibraryItem
 {
 public:
     MusicLibraryItemSong(const Song &s, MusicLibraryItemContainer *parent)
-        : MusicLibraryItem(parent), m_song(s), m_genres(0) { }
+        : MusicLibraryItem(parent), m_song(s) { }
 
-    virtual ~MusicLibraryItemSong();
+    ~MusicLibraryItemSong() override { }
 
-    QString data() const { return m_song.displayTitle(); }
+    QString data() const override { return m_song.displayTitle(); }
     const QString & file() const { return m_song.file; }
-    void setSong(const Song &s);
+    void setSong(const Song &s) { m_song=s; }
     void setFile(const QString &f) { m_song.file=f; }
     quint16 track() const { return m_song.track; }
     void setTrack(quint16 t) { m_song.track=t; }
     void setPlayed(bool p) { m_song.setPlayed(p); }
     quint16 disc() const { return m_song.disc; }
     quint32 time() const { return m_song.time; }
-    const QString & genre() const { return m_song.genre; }
+    QString genre() const { return m_song.firstGenre(); }
     const Song & song() const { return m_song; }
-    Type itemType() const { return Type_Song; }
-    bool hasGenre(const QString &genre) const;
-    QSet<QString> allGenres() const;
-    void setPodcastImage(const QString &img) { m_song.setPodcastImage(img); }
-    bool multipleGenres() const { initGenres(); return m_genres>(void *)1; }
-
-private:
-    void initGenres() const;
+    Type itemType() const override { return Type_Song; }
 
 protected:
     Song m_song;
-    mutable QSet<QString> *m_genres;
 };
 
 #endif
