@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2020 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -63,6 +63,7 @@
 #include <QApplication>
 #include <QMenu>
 #include <QXmlStreamReader>
+#include <algorithm>
 
 GLOBAL_STATIC(PlayQueueModel, instance)
 
@@ -572,7 +573,7 @@ QVariant PlayQueueModel::data(const QModelIndex &index, int role) const
     }
     case Cantata::Role_SubText: {
         const Song &s=songs.at(index.row());
-        return s.artist+QString(" – ")+s.displayAlbum();
+        return s.artist+Song::constSep+s.displayAlbum();
     }
     case Cantata::Role_Time: {
         const Song &s=songs.at(index.row());
@@ -896,6 +897,9 @@ bool PlayQueueModel::dropMimeData(const QMimeData *data,
 
 void PlayQueueModel::load(const QStringList &urls, int action, quint8 priority, bool decreasePriority)
 {
+    if (-1==action) {
+        action = songs.isEmpty() ? MPDConnection::AppendAndPlay : MPDConnection::Append;
+    }
     QStringList useable=parseUrls(urls);
     if (useable.count()) {
         addItems(useable, songs.count(), action, priority, decreasePriority);
@@ -1416,8 +1420,8 @@ static bool artistSort(const Song *s1, const Song *s2)
 
 static bool albumArtistSort(const Song *s1, const Song *s2)
 {
-    const QString v1=s1->hasAlbumArtistSort() ? s1->albumArtistSort() : s1->artistOrComposer();
-    const QString v2=s2->hasAlbumArtistSort() ? s2->albumArtistSort() : s2->artistOrComposer();
+    const QString v1=s1->hasAlbumArtistSort() ? s1->albumArtistSort() : s1->albumArtistOrComposer();
+    const QString v2=s2->hasAlbumArtistSort() ? s2->albumArtistSort() : s2->albumArtistOrComposer();
     int c=v1.localeAwareCompare(v2);
     return c<0 || (c==0 && (*s1)<(*s2));
 }
@@ -1464,23 +1468,23 @@ void PlayQueueModel::sortBy()
         }
 
         if (constSortByArtistKey==key) {
-            qSort(copy.begin(), copy.end(), artistSort);
+            std::sort(copy.begin(), copy.end(), artistSort);
         } else if (constSortByAlbumArtistKey==key) {
-            qSort(copy.begin(), copy.end(), albumArtistSort);
+            std::sort(copy.begin(), copy.end(), albumArtistSort);
         } else if (constSortByAlbumKey==key) {
-            qSort(copy.begin(), copy.end(), albumSort);
+            std::sort(copy.begin(), copy.end(), albumSort);
         } else if (constSortByGenreKey==key) {
-            qSort(copy.begin(), copy.end(), genreSort);
+            std::sort(copy.begin(), copy.end(), genreSort);
         } else if (constSortByYearKey==key) {
-            qSort(copy.begin(), copy.end(), yearSort);
+            std::sort(copy.begin(), copy.end(), yearSort);
         } else if (constSortByComposerKey==key) {
-            qSort(copy.begin(), copy.end(), composerSort);
+            std::sort(copy.begin(), copy.end(), composerSort);
         } else if (constSortByPerformerKey==key) {
-            qSort(copy.begin(), copy.end(), performerSort);
+            std::sort(copy.begin(), copy.end(), performerSort);
         } else if (constSortByTitleKey==key) {
-            qSort(copy.begin(), copy.end(), titleSort);
+            std::sort(copy.begin(), copy.end(), titleSort);
         } else if (constSortByNumberKey==key) {
-            qSort(copy.begin(), copy.end(), trackSort);
+            std::sort(copy.begin(), copy.end(), trackSort);
         }
         QList<quint32> positions;
         for (const Song *s: copy) {
@@ -1608,7 +1612,7 @@ void PlayQueueModel::shuffleAlbums()
     while (!keys.isEmpty()) {
         quint32 key=keys.takeAt(Utils::random(keys.count()));
         QList<const Song *> albumSongs=albums[key];
-        qSort(albumSongs.begin(), albumSongs.end(), songSort);
+        std::sort(albumSongs.begin(), albumSongs.end(), songSort);
         for (const Song *song: albumSongs) {
             positions.append(getRowById(song->id));
         }
