@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2020 Craig Drummond <craig.p.drummond@gmail.com>
  *
  */
 /*
@@ -49,6 +49,7 @@
 #elif defined Q_OS_MAC && defined IOKIT_FOUND
 #include "mac/powermanagement.h"
 #endif
+#include <algorithm>
 #include <QDebug>
 static bool debugEnabled=false;
 #define DBUG if (debugEnabled) qWarning() << "MPDConnection" << QThread::currentThreadId()
@@ -195,6 +196,7 @@ QString MPDConnection::Response::getError(const QByteArray &command)
 MPDConnectionDetails::MPDConnectionDetails()
     : port(6600)
     , dirReadable(false)
+    , applyReplayGain(true)
     , allowLocalStreaming(true)
     , autoUpdate(false)
 {
@@ -232,6 +234,7 @@ MPDConnectionDetails & MPDConnectionDetails::operator=(const MPDConnectionDetail
     streamUrl=o.streamUrl;
     #endif
     replayGain=o.replayGain;
+    applyReplayGain=o.applyReplayGain;
     allowLocalStreaming=o.allowLocalStreaming;
     autoUpdate=o.autoUpdate;
     return *this;
@@ -517,7 +520,7 @@ void MPDConnection::reconnect()
     switch (status) {
     case Success:        
         // Issue #1041 - MPD does not seem to persist user/client made replaygain changes, so use the values read from Cantata's config.
-        if (replaygainSupported() && !details.replayGain.isEmpty()) {
+        if (replaygainSupported() && details.applyReplayGain && !details.replayGain.isEmpty()) {
             sendCommand("replay_gain_mode "+details.replayGain.toLatin1());
         }
         getStatus();
@@ -609,7 +612,7 @@ void MPDConnection::setDetails(const MPDConnectionDetails &d)
         switch (status) {
         case Success:
             // Issue #1041 - MPD does not seem to persist user/client made replaygain changes, so use the values read from Cantata's config.
-            if (replaygainSupported() && !details.replayGain.isEmpty()) {
+            if (replaygainSupported() && details.applyReplayGain && !details.replayGain.isEmpty()) {
                 sendCommand("replay_gain_mode "+details.replayGain.toLatin1());
             }
             serverInfo.detect();
@@ -978,7 +981,7 @@ void MPDConnection::move(const QList<quint32> &items, quint32 pos, quint32 size)
     QList<quint32> moveItems;
 
     moveItems.append(items);
-    qSort(moveItems);
+    std::sort(moveItems.begin(), moveItems.end());
 
     int posOffset = 0;
 
@@ -1340,6 +1343,14 @@ void MPDConnection::goToPrevious()
 {
     toggleStopAfterCurrent(false);
     stopVolumeFade();
+    Response status=sendCommand("status");
+    if (status.ok) {
+        MPDStatusValues sv=MPDParseUtils::parseStatus(status.data);
+        if (sv.timeElapsed>4) {
+            setSeekId(sv.songId, 0);
+            return;
+        }
+    }
     sendCommand("previous");
 }
 
@@ -1866,7 +1877,7 @@ void MPDConnection::removeFromPlaylist(const QString &name, const QList<quint32>
     QList<quint32> sorted=positions;
     QList<quint32> removed;
 
-    qSort(sorted);
+    std::sort(sorted.begin(), sorted.end());
 
     for (int i=sorted.count()-1; i>=0; --i) {
         quint32 idx=sorted.at(i);
@@ -1954,7 +1965,7 @@ void MPDConnection::search(const QString &field, const QString &value, int id)
                     }
                 }
             }
-            qSort(songs);
+            std::sort(songs.begin(), songs.end());
         }
     }
     emit searchResponse(id, songs);
@@ -2024,7 +2035,7 @@ void MPDConnection::listStreams()
 
 void MPDConnection::saveStream(const QString &url, const QString &name)
 {
-    if (sendCommand("playlistadd "+encodeName(constStreamsPlayListName)+" "+encodeName(MPDParseUtils::addStreamName(url, name))).ok) {
+    if (sendCommand("playlistadd "+encodeName(constStreamsPlayListName)+" "+encodeName(MPDParseUtils::addStreamName(url, name, true))).ok) {
         emit savedStream(url, name);
     }
 }
@@ -2039,7 +2050,7 @@ void MPDConnection::removeStreams(const QList<quint32> &positions)
     QList<quint32> sorted=positions;
     QList<quint32> removed;
 
-    qSort(sorted);
+    std::sort(sorted.begin(), sorted.end());
 
     for (int i=sorted.count()-1; i>=0; --i) {
         quint32 idx=sorted.at(i);
@@ -2124,7 +2135,7 @@ bool MPDConnection::doMoveInPlaylist(const QString &name, const QList<quint32> &
     QList<quint32> moveItems=items;
     int posOffset = 0;
 
-    qSort(moveItems);
+    std::sort(moveItems.begin(), moveItems.end());
 
     //first move all items (starting with the biggest) to the end so we don't have to deal with changing rownums
     for (int i = moveItems.size() - 1; i >= 0; i--) {
@@ -2541,16 +2552,20 @@ MpdSocket::~MpdSocket()
 
 void MpdSocket::connectToHost(const QString &hostName, quint16 port, QIODevice::OpenMode mode)
 {
-//     qWarning() << "connectToHost" << hostName << port;
-    if (hostName.startsWith('/') || hostName.startsWith('~')) {
+    DBUG << "connectToHost" << hostName << port;
+    if (hostName.startsWith('/') || hostName.startsWith('~')/* || hostName.startsWith('@')*/) {
         deleteTcp();
         if (!local) {
             local = new QLocalSocket(this);
             connect(local, SIGNAL(stateChanged(QLocalSocket::LocalSocketState)), this, SLOT(localStateChanged(QLocalSocket::LocalSocketState)));
             connect(local, SIGNAL(readyRead()), this, SIGNAL(readyRead()));
         }
-//         qWarning() << "Connecting to LOCAL socket";
-        local->connectToServer(Utils::tildaToHome(hostName), mode);
+        DBUG << "Connecting to LOCAL socket";
+        QString host = Utils::tildaToHome(hostName);
+        /*if ('@'==host[0]) {
+            host[0]='\0';
+        }*/
+        local->connectToServer(host, mode);
     } else {
         deleteLocal();
         if (!tcp) {
@@ -2558,7 +2573,7 @@ void MpdSocket::connectToHost(const QString &hostName, quint16 port, QIODevice::
             connect(tcp, SIGNAL(stateChanged(QAbstractSocket::SocketState)), this, SIGNAL(stateChanged(QAbstractSocket::SocketState)));
             connect(tcp, SIGNAL(readyRead()), this, SIGNAL(readyRead()));
         }
-//         qWarning() << "Connecting to TCP socket";
+        DBUG << "Connecting to TCP socket";
         tcp->connectToHost(hostName, port, mode);
     }
 }
