@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2020 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -35,6 +35,7 @@
 #include "support/monoicon.h"
 #include "widgets/textbrowser.h"
 #include "support/messagewidget.h"
+#include "support/flattoolbutton.h"
 #include "gui/covers.h"
 #include "rssparser.h"
 #include "podcastservice.h"
@@ -47,6 +48,7 @@
 #include <QHeaderView>
 #include <QStringList>
 #include <QFile>
+#include <QFileDialog>
 #include <QXmlStreamReader>
 #include <QCryptographicHash>
 #include <QCache>
@@ -220,6 +222,30 @@ void PodcastPage::addPodcast(const QString &title, const QUrl &url, const QUrl &
     podItem->setIcon(0, Icons::self()->audioListIcon);
 }
 
+void PodcastPage::addCategory(const OpmlParser::Category &cat, QTreeWidgetItem *p)
+{
+    if (cat.categories.isEmpty() && cat.podcasts.isEmpty()) {
+        return;
+    }
+
+    QTreeWidgetItem *catItem=p ? new QTreeWidgetItem(p, QStringList() << cat.name)
+                               : new QTreeWidgetItem(tree, QStringList() << cat.name);
+
+    catItem->setData(0, IsPodcastRole, false);
+    catItem->setIcon(0, Icons::self()->folderListIcon);
+    for (const OpmlParser::Podcast &pod: cat.podcasts) {
+        addPodcast(pod, catItem);
+    }
+    for (const OpmlParser::Category &subCat: cat.categories) {
+        addCategory(subCat, catItem);
+    }
+}
+
+void PodcastPage::addPodcast(const OpmlParser::Podcast &pod, QTreeWidgetItem *p)
+{
+    addPodcast(pod.name, pod.url, pod.image, pod.description, pod.htmlUrl, p);
+}
+
 static QString encode(const QImage &img)
 {
     QByteArray bytes;
@@ -290,7 +316,7 @@ void PodcastPage::imageJobFinished()
         if (img.width()>maxImageSize || img.height()>maxImageSize) {
             img=img.scaled(maxImageSize, maxImageSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
         }
-        imageCache.insert(imageJob->property(constOrigUrlProperty).toUrl(), new QImage(img), img.byteCount());
+        imageCache.insert(imageJob->property(constOrigUrlProperty).toUrl(), new QImage(img), img.sizeInBytes());
         updateText();
     }
     imageJob=nullptr;
@@ -471,30 +497,6 @@ void OpmlBrowsePage::parseResonse(QIODevice *dev)
     }
 }
 
-void OpmlBrowsePage::addCategory(const OpmlParser::Category &cat, QTreeWidgetItem *p)
-{
-    if (cat.categories.isEmpty() && cat.podcasts.isEmpty()) {
-        return;
-    }
-
-    QTreeWidgetItem *catItem=p ? new QTreeWidgetItem(p, QStringList() << cat.name)
-                               : new QTreeWidgetItem(tree, QStringList() << cat.name);
-
-    catItem->setData(0, IsPodcastRole, false);
-    catItem->setIcon(0, Icons::self()->folderListIcon);
-    for (const OpmlParser::Podcast &pod: cat.podcasts) {
-        addPodcast(pod, catItem);
-    }
-    for (const OpmlParser::Category &subCat: cat.categories) {
-        addCategory(subCat, catItem);
-    }
-}
-
-void OpmlBrowsePage::addPodcast(const OpmlParser::Podcast &pod, QTreeWidgetItem *p)
-{
-    PodcastPage::addPodcast(pod.name, pod.url, pod.image, pod.description, pod.htmlUrl, p);
-}
-
 PodcastUrlPage::PodcastUrlPage(QWidget *p)
     : PodcastPage(p, tr("URL"))
 {
@@ -506,18 +508,23 @@ PodcastUrlPage::PodcastUrlPage(QWidget *p)
     mainLayout->setMargin(0);
     urlEntry=new LineEdit(p);
     urlEntry->setPlaceholderText(tr("Enter podcast URL..."));
-    loadButton=new QPushButton(tr("Load"), p);
+    QPushButton *loadButton=new QPushButton(tr("Load"), p);
+    FlatToolButton *pathReq=new FlatToolButton(p);
+    pathReq->setIcon(MonoIcon::icon(FontAwesome::foldero, Utils::monoIconColor()));
+    pathReq->setToolTip(tr("Load local podcast file"));
     QWidget::setTabOrder(urlEntry, loadButton);
     QWidget::setTabOrder(loadButton, tree);
     searchLayout->addWidget(urlEntry);
     searchLayout->addWidget(loadButton);
+    searchLayout->addWidget(pathReq);
     viewLayout->addWidget(tree, 1);
     viewLayout->addWidget(text, 0);
-    mainLayout->addWidget(new QLabel(tr("Enter podcast URL below, and press 'Load'"), this));
+    mainLayout->addWidget(new QLabel(tr("Enter podcast URL below, and press 'Load', or press the folder icon to load a local podcast file."), this));
     mainLayout->addLayout(searchLayout);
     mainLayout->addLayout(viewLayout);
     connect(urlEntry, SIGNAL(returnPressed()), SLOT(loadUrl()));
     connect(loadButton, SIGNAL(clicked()), SLOT(loadUrl()));
+    connect(pathReq, SIGNAL(clicked()), SLOT(openPath()));
     icn=Icons::self()->rssListIcon;
 }
 
@@ -547,22 +554,56 @@ void PodcastUrlPage::loadUrl()
     }
 }
 
+void PodcastUrlPage::openPath()
+{
+    QString path = QFileDialog::getOpenFileName(this, QString(), QDir::homePath(), tr("Podcasts (*.xml, *.rss, *.opml"));
+
+    if (!path.isEmpty()) {
+        QFile f(path);
+        if (f.open(QIODevice::ReadOnly|QIODevice::Text)) {
+            parse(&f);
+        } else {
+            emit error(tr("Failed to read file!"));
+        }
+    }
+}
 void PodcastUrlPage::parseResonse(QIODevice *dev)
 {
     if (!dev) {
         emit error(tr("Failed to fetch podcast!"));
         return;
     }
-    RssParser::Channel ch=RssParser::parse(dev, false, true);
-    if (!ch.isValid()) {
-        emit error(tr("Failed to parse podcast."));
-        return;
+    parse(dev);
+}
+
+void PodcastUrlPage::parse(QIODevice *dev)
+{
+    auto data = dev->readAll();
+    QBuffer buf(&data);
+    buf.open(QIODevice::ReadOnly|QIODevice::Text);
+    RssParser::Channel ch=RssParser::parse(&buf, false, true);
+    if (ch.isValid()) {
+        if (ch.video) {
+            emit error(tr("Cantata only supports audio podcasts! The URL entered contains only video podcasts."));
+        } else {
+            addPodcast(ch.name, currentUrl, ch.image, ch.description, QString(), nullptr);
+        }
+    } else {
+        OpmlParser::Category parsed=OpmlParser::parse(data);
+        if (parsed.categories.isEmpty() && parsed.podcasts.isEmpty()) {
+            emit error(tr("Failed to parse podcast."));
+        } else {
+            if (1==parsed.categories.count() && parsed.podcasts.isEmpty()) {
+                parsed=parsed.categories.at(0);
+            }
+            for (const OpmlParser::Category &cat: parsed.categories) {
+                addCategory(cat, nullptr);
+            }
+            for (const OpmlParser::Podcast &pod: parsed.podcasts) {
+                addPodcast(pod, nullptr);
+            }
+        }
     }
-    if (!ch.isValid()) {
-        emit error(tr("Cantata only supports audio podcasts! The URL entered contains only video podcasts."));
-        return;
-    }
-    addPodcast(ch.name, currentUrl, ch.image, ch.description, QString(), nullptr);
 }
 
 int PodcastSearchDialog::instanceCount()
