@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2020 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -24,6 +24,7 @@
 #include "podcastservice.h"
 #include "podcastsettingsdialog.h"
 #include "rssparser.h"
+#include "support/globalstatic.h"
 #include "support/utils.h"
 #include "support/monoicon.h"
 #include "gui/settings.h"
@@ -44,8 +45,26 @@
 #include <QXmlStreamWriter>
 #include <QCryptographicHash>
 #include <QMimeData>
+#include <QDateTime>
+#include <QTextDocument>
 #include <stdio.h>
 
+GLOBAL_STATIC(PodcastService, instance)
+
+static QString encodeName(const QString &name)
+{
+    QString n=name;
+    n=n.replace("/", "_");
+    n=n.replace("\\", "_");
+    n=n.replace(":", "_");
+    return n;
+}
+
+static inline QString episodeFileName(const QUrl &url)
+{
+    return url.path().split('/', QString::SkipEmptyParts).join('_').replace('~', '_');
+}
+
 PodcastService::Proxy::Proxy(QObject *parent)
     : ProxyModel(parent)
     , unplayedOnly(false)
@@ -198,8 +217,10 @@ Song PodcastService::Episode::toSong() const
     Song song;
     song.title=name;
     song.file=url.toString();
-    song.artist=parent->name;
+    song.album=parent->name;
     song.time=duration;
+    song.setExtraField(Song::OnlineImageUrl, parent->imageUrl.toString());
+    song.setExtraField(Song::OnlineImageCacheName, parent->imageFile);
     if (!localFile.isEmpty()) {
         song.setLocalPath(localFile);
     }
@@ -242,6 +263,8 @@ bool PodcastService::Podcast::load()
 
     QXmlStreamReader reader(&compressor);
     unplayedCount=0;
+
+    QString podPath=Settings::self()->podcastDownloadPath();
     while (!reader.atEnd()) {
         reader.readNext();
         if (!reader.error() && reader.isStartElement()) {
@@ -256,6 +279,9 @@ bool PodcastService::Podcast::load()
                 if (url.isEmpty() || name.isEmpty()) {
                     return false;
                 }
+                if (!podPath.isEmpty()) {
+                    podPath=Utils::fixPath(podPath)+Utils::fixPath(encodeName(name));
+                }
             } else if (constEpisodeTag == element) {
                 QString epName=attributes.value(constNameAttribute).toString();
                 QString epUrl=attributes.value(constUrlAttribute).toString();
@@ -269,6 +295,11 @@ bool PodcastService::Podcast::load()
                     ep->descr=attributes.value(constDescrAttribute).toString();
                     if (QFile::exists(localFile)) {
                         ep->localFile=localFile;
+                    } else if (!podPath.isEmpty()) {
+                        QString localPath=podPath+episodeFileName(ep->url);
+                        if (QFile::exists(localPath)) {
+                            ep->localFile = localPath;
+                        }
                     }
 
                     episodes.append(ep);
@@ -385,8 +416,8 @@ const Song & PodcastService::Podcast::coverSong()
     return song;
 }
 
-PodcastService::PodcastService(QObject *p)
-    : ActionModel(p)
+PodcastService::PodcastService()
+    : ActionModel(nullptr)
     , downloadJob(nullptr)
     , rssUpdateTimer(nullptr)
 {
@@ -412,6 +443,44 @@ QString PodcastService::descr() const
     return tr("Subscribe to RSS feeds");
 }
 
+bool PodcastService::episodeCover(const Song &s, QImage &img, QString &imgFilename) const
+{
+    if ((s.isFromOnlineService() && s.onlineService()==constName) || isPodcastFile(s.file)) {
+        QString path=s.decodedPath();
+        if (path.isEmpty()) {
+            path=s.file;
+        }
+        for (Podcast *podcast: podcasts) {
+            for (Episode *episode: podcast->episodes) {
+                if (episode->url==path || episode->localFile==path) {
+                    imgFilename = podcast->imageFile;
+                    img = QImage(imgFilename);
+                    return true;
+                }
+            }
+        }
+    }
+    return false;
+}
+
+QString PodcastService::episodeDescr(const Song &s) const
+{
+    if ((s.isFromOnlineService() && s.onlineService()==constName) || isPodcastFile(s.file)) {
+        QString path=s.decodedPath();
+        if (path.isEmpty()) {
+            path=s.file;
+        }
+        for (Podcast *podcast: podcasts) {
+            for (Episode *episode: podcast->episodes) {
+                if (episode->url==path || episode->localFile==path) {
+                    return episode->descr.replace("<p><br></p>", "");
+                }
+            }
+        }
+    }
+    return QString();
+}
+
 int PodcastService::rowCount(const QModelIndex &index) const
 {
     if (index.column()>0) {
@@ -473,12 +542,14 @@ QModelIndex PodcastService::index(int row, int col, const QModelIndex &parent) c
     return row<podcasts.count() ? createIndex(row, col, podcasts.at(row)) : QModelIndex();
 }
 
-static QString trimDescr(QString descr)
+static QString trimDescr(QString descr, int limit=1000)
 {
     if (!descr.isEmpty()) {
-        static const int constMaxDescrLen=300;
-        if (descr.length()>constMaxDescrLen) {
-            descr=descr.left(constMaxDescrLen)+QLatin1String("...");
+        QTextDocument doc;
+        doc.setHtml(descr);
+        descr=doc.toPlainText().simplified();
+        if (descr.length()>limit) {
+            descr=descr.left(limit)+QLatin1String("...");
         }
         descr+=QLatin1String("<br/><br/>");
     }
@@ -757,10 +828,23 @@ void PodcastService::rssJobFinished()
             podcast->name=ch.name;
             podcast->descr=ch.description;
             podcast->unplayedCount=ch.episodes.count();
+
+            QString podPath=Settings::self()->podcastDownloadPath();
+            if (!podPath.isEmpty()) {
+                podPath=Utils::fixPath(podPath)+Utils::fixPath(encodeName(podcast->name));
+            }
+
             for (const RssParser::Episode &ep: ch.episodes) {
                 Episode *episode=new Episode(ep.publicationDate, ep.name, ep.url, podcast);
                 episode->duration=ep.duration;
                 episode->descr=ep.description;
+                if (!podPath.isEmpty()) {
+                    // Check if we had subscribed to this before, and downloaded episodes...
+                    QString localPath=podPath+episodeFileName(episode->url);
+                    if (QFile::exists(localPath)) {
+                        episode->localFile = localPath;
+                    }
+                }
                 podcast->add(episode);
             }
             podcast->save();
@@ -1006,15 +1090,6 @@ void PodcastService::setPodcastsAsListened(Podcast *pod, const QList<Episode *>
     }
 }
 
-static QString encodeName(const QString &name)
-{
-    QString n=name;
-    n=n.replace("/", "_");
-    n=n.replace("\\", "_");
-    n=n.replace(":", "_");
-    return n;
-}
-
 void PodcastService::downloadEpisode(const Podcast *podcast, const QUrl &episode)
 {
     QString dest=Settings::self()->podcastDownloadPath();
@@ -1024,7 +1099,7 @@ void PodcastService::downloadEpisode(const Podcast *podcast, const QUrl &episode
     if (downloadingEpisode(episode)) {
         return;
     }
-    dest=Utils::fixPath(dest)+Utils::fixPath(encodeName(podcast->name))+Utils::getFile(episode.path());
+    dest=Utils::fixPath(dest)+Utils::fixPath(encodeName(podcast->name))+episodeFileName(episode);
     toDownload.append(DownloadEntry(episode, podcast->url, dest));
     updateEpisode(podcast->url, episode, Episode::QueuedForDownload);
     doNextDownload();
@@ -1247,6 +1322,40 @@ void PodcastService::stopRssUpdateTimer()
     }
 }
 
+bool PodcastService::exportSubscriptions(const QString &name)
+{
+    QFile f(name);
+    if (!f.open(QIODevice::Text|QIODevice::WriteOnly)) {
+        return false;
+    }
+    QString date = QDateTime::currentDateTime().toString(Qt::RFC2822Date);
+    QXmlStreamWriter writer(&f);
+    writer.writeStartDocument();
+    writer.writeStartElement(QLatin1String("opml"));
+    writer.writeAttribute(QLatin1String("version"), QLatin1String("1.1"));
+    writer.writeStartElement(QLatin1String("head"));
+    writer.writeTextElement(QLatin1String("title"), QLatin1String("Cantata Podcasts"));
+    writer.writeTextElement(QLatin1String("dateCreated"), date);
+    writer.writeTextElement(QLatin1String("dateModified"), date);
+
+    writer.writeEndElement(); // head
+    writer.writeStartElement(QLatin1String("body"));
+    for (Podcast *podcast: podcasts) {
+        writer.writeStartElement(QLatin1String("outline"));
+        writer.writeAttribute(QLatin1String("text"), podcast->name);
+        writer.writeAttribute(QLatin1String("description"), podcast->descr);
+        writer.writeAttribute(QLatin1String("type"), QLatin1String("rss"));
+        writer.writeAttribute(QLatin1String("xmlUrl"), podcast->url.toEncoded());
+        writer.writeAttribute(QLatin1String("imageUrl"), podcast->imageUrl.toEncoded());
+        writer.writeEndElement(); // outline
+    }
+    writer.writeEndElement(); // body
+    writer.writeEndElement(); // opml
+    writer.writeEndDocument();
+    f.close();
+    return true;
+}
+
 void PodcastService::updateRss()
 {
     for (Podcast *podcast: podcasts) {
@@ -1277,6 +1386,7 @@ void PodcastService::currentMpdSong(const Song &s)
                         idx=createIndex(podcasts.indexOf(podcast), 0, (void *)podcast);
                         emit dataChanged(idx, idx);
                     }
+
                     return;
                 }
             }
