@@ -23,31 +23,20 @@
 #include "action.h"
 #include "config.h"
 #include "gtkstyle.h"
+#include "utils.h"
 #include <QApplication>
 #include <QKeySequence>
 
 Action::Action(QObject *parent)
-#ifdef ENABLE_KDE_SUPPORT
-: KAction(parent)
-#else
-: QAction(parent)
-#endif
+    : QAction(parent)
 {
-  #ifndef ENABLE_KDE_SUPPORT
   init();
-  #endif
 }
 
 Action::Action(const QString &text, QObject *parent, const QObject *receiver, const char *slot, const QKeySequence &shortcut)
-#ifdef ENABLE_KDE_SUPPORT
-: KAction(parent)
-#else
-: QAction(parent)
-#endif
+    : QAction(parent)
 {
-  #ifndef ENABLE_KDE_SUPPORT
   init();
-  #endif
   setText(text);
   setShortcut(shortcut);
   if(receiver && slot)
@@ -55,15 +44,9 @@ Action::Action(const QString &text, QObject *parent, const QObject *receiver, co
 }
 
 Action::Action(const QIcon &icon, const QString &text, QObject *parent, const QObject *receiver, const char *slot, const QKeySequence &shortcut)
-#ifdef ENABLE_KDE_SUPPORT
-: KAction(parent)
-#else
-: QAction(parent)
-#endif
+    : QAction(parent)
 {
-  #ifndef ENABLE_KDE_SUPPORT
   init();
-  #endif
   setIcon(icon);
   setText(text);
   setShortcut(shortcut);
@@ -73,11 +56,9 @@ Action::Action(const QIcon &icon, const QString &text, QObject *parent, const QO
 
 void Action::initIcon(QAction *act)
 {
-    #ifndef ENABLE_UBUNTU
     if (GtkStyle::isActive() && act) {
         act->setIconVisibleInMenu(false);
     }
-    #endif
 }
 
 static const char *constPlainToolTipProperty="plain-tt";
@@ -89,8 +70,11 @@ void Action::updateToolTip(QAction *act)
     }
     QKeySequence sc=act->shortcut();
     if (sc.isEmpty()) {
-        act->setToolTip(act->property(constPlainToolTipProperty).toString());
-        act->setProperty(constPlainToolTipProperty, QString());
+        QString tt=act->property(constPlainToolTipProperty).toString();
+        if (!tt.isEmpty()) {
+            act->setToolTip(tt);
+            act->setProperty(constPlainToolTipProperty, QString());
+        }
     } else {
         QString tt=act->property(constPlainToolTipProperty).toString();
         if (tt.isEmpty()) {
@@ -99,11 +83,19 @@ void Action::updateToolTip(QAction *act)
         }
         act->setToolTip(QString::fromLatin1("%1 <span style=\"color: gray; font-size: small\">%2</span>")
                         .arg(tt)
-                        .arg(sc.toString()));
+                        .arg(sc.toString(QKeySequence::NativeText)));
     }
 }
 
-#ifndef ENABLE_KDE_SUPPORT
+static const char * constSettingsText="tt-for-settings";
+
+QString Action::settingsText(QAction *act)
+{
+    return act->property(constSettingsText).isValid()
+            ? act->property(constSettingsText).toString()
+            : Utils::stripAcceleratorMarkers(act->text());
+}
+
 void Action::init() {
   connect(this, SIGNAL(triggered(bool)), this, SLOT(slotTriggered()));
 
@@ -122,6 +114,14 @@ void Action::setShortcutConfigurable(bool b) {
   setProperty("isShortcutConfigurable", b);
 }
 
+void Action::setSettingsText(const QString &text) {
+    setProperty(constSettingsText, text);
+}
+
+void Action::setSettingsText(Action *parent) {
+    setSettingsText(Utils::strippedText(parent->text())+QLatin1String(" / ")+Utils::strippedText(text()));
+}
+
 QKeySequence Action::shortcut(ShortcutTypes type) const {
   Q_ASSERT(type);
   if(type == DefaultShortcut)
@@ -145,4 +145,4 @@ void Action::setShortcut(const QKeySequence &key, ShortcutTypes type) {
     QAction::setShortcut(key);
 }
 
-#endif // !ENABLE_KDE_SUPPORT
+#include "moc_action.cpp"
