@@ -24,10 +24,12 @@
 #include "action.h"
 #include "icon.h"
 #include <QCoreApplication>
+#include <QSettings>
+#include <QMetaMethod>
 
 static const char *constProp="Category";
-static ActionCollection *coll=0;
-static QWidget *mainWidget=0;
+static ActionCollection *coll=nullptr;
+static QWidget *mainWidget=nullptr;
 void ActionCollection::setMainWidget(QWidget *w)
 {
     mainWidget=w;
@@ -49,13 +51,8 @@ Action * ActionCollection::createAction(const QString &name, const QString &text
 {
     Action *act = static_cast<Action *>(addAction(name));
     act->setText(text);
-    if (0!=icon) {
-        #ifndef ENABLE_KDE_SUPPORT
-        if ('m'==icon[0] && 'e'==icon[1] && 'd'==icon[2] && 'i'==icon[3] && 'a'==icon[4] && '-'==icon[5]) {
-            act->setIcon(Icon::getMediaIcon(icon));
-        } else
-        #endif
-        act->setIcon(Icon(icon));
+    if (nullptr!=icon) {
+        act->setIcon(Icon::get(icon));
     }
     if (!whatsThis.isEmpty()) {
         act->setWhatsThis(whatsThis);
@@ -80,7 +77,7 @@ Action * ActionCollection::createAction(const QString &name, const QString &text
 
 void ActionCollection::updateToolTips()
 {
-    foreach (QAction *act, actions()) {
+    for (QAction *act: actions()) {
         QString prev=act->toolTip();
         Action::updateToolTip(act);
         if (prev!=act->toolTip()) {
@@ -89,10 +86,6 @@ void ActionCollection::updateToolTips()
     }
 }
 
-#ifndef ENABLE_KDE_SUPPORT
-
-#include <QSettings>
-
 ActionCollection::ActionCollection(QObject *parent) : QObject(parent) {
   _connectTriggered = _connectHovered = false;
 }
@@ -107,7 +100,7 @@ void ActionCollection::clear() {
 }
 
 QAction *ActionCollection::action(const QString &name) const {
-  return _actionByName.value(name, 0);
+  return _actionByName.value(name, nullptr);
 }
 
 QList<QAction *> ActionCollection::actions() const {
@@ -116,7 +109,7 @@ QList<QAction *> ActionCollection::actions() const {
 
 Action *ActionCollection::addAction(const QString &name, Action *action) {
   QAction *act = addAction(name, static_cast<QAction *>(action));
-  Q_UNUSED(act);
+  Q_UNUSED(act)
   Q_ASSERT(act == action);
   return action;
 }
@@ -143,7 +136,7 @@ QAction *ActionCollection::addAction(const QString &name, QAction *action) {
     indexName = indexName.sprintf("unnamed-%p", (void *)action);
 
   // do we already have this action?
-  if(_actionByName.value(indexName, 0) == action)
+  if(_actionByName.value(indexName, nullptr) == action)
     return action;
   // or maybe another action under this name?
   if(QAction *oldAction = _actionByName.value(indexName))
@@ -160,7 +153,7 @@ QAction *ActionCollection::addAction(const QString &name, QAction *action) {
   _actionByName.insert(indexName, action);
   _actions.append(action);
 
-  foreach(QWidget *widget, _associatedWidgets) {
+  for (QWidget *widget: _associatedWidgets) {
     widget->addAction(action);
   }
 
@@ -180,9 +173,9 @@ void ActionCollection::removeAction(QAction *action) {
 
 QAction *ActionCollection::takeAction(QAction *action) {
   if(!unlistAction(action))
-    return 0;
+    return nullptr;
 
-  foreach(QWidget *widget, _associatedWidgets) {
+  for (QWidget *widget: _associatedWidgets) {
     widget->removeAction(action);
   }
 
@@ -204,7 +197,7 @@ void ActionCollection::readSettings() {
   s.beginGroup(configKey());
   QStringList savedShortcuts = s.childKeys();
 
-  foreach(const QString &name, _actionByName.keys()) {
+  for (const QString &name: _actionByName.keys()) {
     if(!savedShortcuts.contains(name))
       continue;
     Action *action = qobject_cast<Action *>(_actionByName.value(name));
@@ -216,7 +209,7 @@ void ActionCollection::readSettings() {
 void ActionCollection::writeSettings() const {
   QSettings s;
   s.beginGroup(configKey());
-  foreach(const QString &name, _actionByName.keys()) {
+  for (const QString &name: _actionByName.keys()) {
     Action *action = qobject_cast<Action *>(_actionByName.value(name));
     if(!action)
       continue;
@@ -249,33 +242,6 @@ void ActionCollection::actionDestroyed(QObject *obj) {
   unlistAction(action);
 }
 
-void ActionCollection::connectNotify(const char *signal) {
-  #if QT_VERSION < 0x050000
-  if(_connectHovered && _connectTriggered)
-    return;
-
-  if(QMetaObject::normalizedSignature(SIGNAL(actionHovered(QAction*))) == signal) {
-    if(!_connectHovered) {
-      _connectHovered = true;
-      foreach (QAction* action, actions())
-        connect(action, SIGNAL(hovered()), SLOT(slotActionHovered()));
-    }
-  } else if(QMetaObject::normalizedSignature(SIGNAL(actionTriggered(QAction*))) == signal) {
-    if(!_connectTriggered) {
-      _connectTriggered = true;
-      foreach (QAction* action, actions())
-        connect(action, SIGNAL(triggered(bool)), SLOT(slotActionTriggered()));
-    }
-  }
-
-  QObject::connectNotify(signal);
-  #else
-  Q_UNUSED(signal)
-  #endif
-}
-
-#if QT_VERSION >= 0x050000
-#include <QMetaMethod>
 void ActionCollection::connectNotify(const QMetaMethod &signal) {
   if(_connectHovered && _connectTriggered)
     return;
@@ -283,23 +249,22 @@ void ActionCollection::connectNotify(const QMetaMethod &signal) {
   if(QMetaObject::normalizedSignature(SIGNAL(actionHovered(QAction*))) == signal.methodSignature()) {
     if(!_connectHovered) {
       _connectHovered = true;
-      foreach (QAction* action, actions())
+      for (QAction* action: actions())
         connect(action, SIGNAL(hovered()), SLOT(slotActionHovered()));
     }
   } else if(QMetaObject::normalizedSignature(SIGNAL(actionTriggered(QAction*))) == signal.methodSignature()) {
     if(!_connectTriggered) {
       _connectTriggered = true;
-      foreach (QAction* action, actions())
+      for (QAction* action: actions())
         connect(action, SIGNAL(triggered(bool)), SLOT(slotActionTriggered()));
     }
   }
 
   QObject::connectNotify(signal);
 }
-#endif
 
 void ActionCollection::associateWidget(QWidget *widget) const {
-  foreach(QAction *action, actions()) {
+  for (QAction *action: actions()) {
     if(!widget->actions().contains(action))
       widget->addAction(action);
   }
@@ -314,7 +279,7 @@ void ActionCollection::addAssociatedWidget(QWidget *widget) {
 }
 
 void ActionCollection::removeAssociatedWidget(QWidget *widget) {
-  foreach(QAction *action, actions())
+  for (QAction *action: actions())
     widget->removeAction(action);
   _associatedWidgets.removeAll(widget);
   disconnect(widget, SIGNAL(destroyed(QObject *)), this, SLOT(associatedWidgetDestroyed(QObject *)));
@@ -325,8 +290,8 @@ QList<QWidget *> ActionCollection::associatedWidgets() const {
 }
 
 void ActionCollection::clearAssociatedWidgets() {
-  foreach(QWidget *widget, _associatedWidgets)
-    foreach(QAction *action, actions())
+  for (QWidget *widget: _associatedWidgets)
+    for (QAction *action: actions())
       widget->removeAction(action);
 
   _associatedWidgets.clear();
@@ -351,4 +316,4 @@ bool ActionCollection::unlistAction(QAction *action) {
   return true;
 }
 
-#endif /* HAVE_KDE */
+#include "moc_actioncollection.cpp"
