@@ -23,49 +23,26 @@
 #ifndef ACTIONCOLLECTION_H_
 #define ACTIONCOLLECTION_H_
 
-class Action;
-class QIcon;
-
-#ifdef ENABLE_KDE_SUPPORT
-#include <KDE/KActionCollection>
-
-class ActionCollection : public KActionCollection {
-    Q_OBJECT
-
-public:
-    explicit ActionCollection(QObject *parent) : KActionCollection(parent) {};
-
-    static void setMainWidget(QWidget *w);
-    static ActionCollection * get();
-    Action * createAction(const QString &name, const QString &text, const char *icon=0, const QString &whatsThis=QString());
-    Action * createAction(const QString &name, const QString &text, const QIcon &icon, const QString &whatsThis=QString());
-
-    void updateToolTips();
-
-Q_SIGNALS:
-    void tooltipUpdated(QAction *);
-};
-
-#else
-
 #include <QList>
 #include <QMap>
 #include <QObject>
 #include <QString>
 
-class QWidget;
+class Action;
 class QAction;
+class QWidget;
+class QIcon;
 
 class ActionCollection : public QObject {
     Q_OBJECT
 
 public:
     explicit ActionCollection(QObject *parent);
-    virtual ~ActionCollection();
+    ~ActionCollection() override;
 
     static void setMainWidget(QWidget *w);
     static ActionCollection * get();
-    Action * createAction(const QString &name, const QString &text, const char *icon=0, const QString &whatsThis=QString());
+    Action * createAction(const QString &name, const QString &text, const char *icon=nullptr, const QString &whatsThis=QString());
     Action * createAction(const QString &name, const QString &text, const QIcon &icon, const QString &whatsThis=QString());
 
     /// Clears the entire action collection, deleting all actions.
@@ -98,13 +75,13 @@ public:
 
     QAction *addAction(const QString &name, QAction *action);
     Action *addAction(const QString &name, Action *action);
-    Action *addAction(const QString &name, const QObject *receiver = 0, const char *member = 0);
+    Action *addAction(const QString &name, const QObject *receiver = nullptr, const char *member = nullptr);
     void removeAction(QAction *action);
     QAction *takeAction(QAction *action);
 
     /// Create new action under the given name, add it to the collection and connect its triggered(bool) signal to the specified receiver.
     template<class ActionType>
-    ActionType *add(const QString &name, const QObject *receiver = 0, const char *member = 0) {
+    ActionType *add(const QString &name, const QObject *receiver = nullptr, const char *member = nullptr) {
         ActionType *a = new ActionType(this);
         if(receiver && member)
             connect(a, SIGNAL(triggered(bool)), receiver, member);
@@ -121,13 +98,10 @@ Q_SIGNALS:
     void tooltipUpdated(QAction *);
 
 protected Q_SLOTS:
-    virtual void connectNotify(const char *signal);
     virtual void slotActionTriggered();
 
-#if QT_VERSION >= 0x050000
 protected:
-    virtual void connectNotify(const QMetaMethod &signal);
-#endif
+    void connectNotify(const QMetaMethod &signal) override;
 
 private Q_SLOTS:
     void slotActionHovered();
@@ -147,5 +121,3 @@ private:
 };
 
 #endif
-
-#endif
