@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2020 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -36,6 +36,7 @@
 #include <QEventLoop>
 #include <QStandardPaths>
 #include <QSet>
+#include <QUrl>
 #ifndef _MSC_VER 
 #include <unistd.h>
 #include <utime.h>
@@ -59,12 +60,13 @@ const QLatin1String Utils::constDirSepStr("/");
 const char * Utils::constDirSepCharStr="/";
 
 static const QLatin1String constHttp("http:");
+static const QLatin1String constHttps("https:");
 
 QString Utils::fixPath(const QString &dir, bool ensureEndsInSlash)
 {
     QString d(dir);
 
-    if (!d.isEmpty() && !d.startsWith(constHttp)) {
+    if (!d.isEmpty() && !d.startsWith(constHttp) && !d.startsWith(constHttps)) {
         #ifdef Q_OS_WIN
         // Windows shares can be \\host\share (which gets converted to //host/share)
         // so if th epath starts with // we need to keep this double slash.
@@ -110,19 +112,25 @@ QString Utils::tildaToHome(const QString &s)
 }
 #endif
 
-QString Utils::getDir(const QString &file)
+QString Utils::getDir(const QString &file, bool addSlash)
 {
     bool isCueFile=file.contains("/cue:///") && file.contains("?pos=");
     QString d(file);
     int slashPos(d.lastIndexOf(constDirSep));
 
-    if(slashPos!=-1) {
+    if (slashPos!=-1) {
         d.remove(slashPos+1, d.length());
     }
 
     if (isCueFile) {
         d.remove("cue:///");
     }
+    if (!addSlash) {
+        if (d.endsWith("/")) {
+            return d.left(d.length()-1);
+        }
+        return d;
+    }
     return fixPath(d);
 }
 
@@ -223,9 +231,44 @@ QString Utils::stripAcceleratorMarkers(QString label)
     return label;
 }
 
+QMap<QString, QString> Utils::hashParams(const QString &url)
+{
+    QMap<QString, QString> map;
+    int start=url.indexOf("#");
+    if (start>0) {
+        QStringList parts = url.mid(start+1).split("&");
+        for (const QString &p: parts) {
+            QStringList kv = p.split("=");
+            if (kv.length()>=2) {
+                QString key = kv[0];
+                kv.removeAt(0);
+                map.insert(key, QUrl::fromPercentEncoding(kv.join("=").toLatin1()));
+            } else {
+                map.insert(QString("-"), QUrl::fromPercentEncoding(p.toLatin1()));
+            }
+        }
+    }
+    return map;
+}
+
+QString Utils::addHashParam(const QString &url, const QString &key, const QString &val)
+{
+    if (val.isEmpty()) {
+        return url;
+    }
+
+    return url+(url.contains('#') ? "&" : "#")+(key.isEmpty() ? "" : (key+"="))+QUrl::toPercentEncoding(val);
+}
+
+QString Utils::removeHash(const QString &url)
+{
+    int hash=url.indexOf('#');
+    return hash<0 ? url : url.left(hash);
+}
+
 QString Utils::convertPathForDisplay(const QString &path, bool isFolder)
 {
-    if (path.isEmpty() || path.startsWith(constHttp)) {
+    if (path.isEmpty() || path.startsWith(constHttp) || path.startsWith(constHttps)) {
         return path;
     }
 
@@ -246,7 +289,7 @@ QString Utils::convertPathFromDisplay(const QString &path, bool isFolder)
         return p;
     }
 
-    if (p.startsWith(constHttp)) {
+    if (p.startsWith(constHttp) || p.startsWith(constHttps)) {
         return fixPath(p);
     }
     return tildaToHome(fixPath(QDir::fromNativeSeparators(p), isFolder));
