@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2020 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -172,14 +172,14 @@ static QString streamText(const Song &song, const QString &trackTitle, bool useN
                   ? song.title
                   : song.title.isEmpty()
                     ? songName
-                    : (song.title + QString(" – ") + songName);
+                    : (song.title + Song::constSep + songName);
     } else if (!song.title.isEmpty() && !song.artist.isEmpty()) {
         QString name=song.name();
-        return song.artist + QString(" – ") + (!useName || name.isEmpty()
+        return song.artist + Song::constSep + (!useName || name.isEmpty()
                                         ? song.title
                                         : song.title.isEmpty()
                                             ? name
-                                            : (song.title + QString(" – ") + name));
+                                            : (song.title + Song::constSep + name));
     } else {
         return trackTitle;
     }
@@ -296,13 +296,13 @@ void GroupedViewDelegate::paint(QPainter *painter, const QStyleOptionViewItem &o
             title=Song::unknown();
             track=trackTitle;
         } else if (song.album.isEmpty()) {
-            title=song.artistOrComposer();
+            title=song.albumArtistOrComposer();
             track=song.trackAndTitleStr();
         } else {
             if (song.isFromOnlineService()) {
                 title=Song::displayAlbum(song.albumName(), Song::albumYear(song));
             } else {
-                title=song.artistOrComposer()+QString(" – ")+Song::displayAlbum(song.albumName(), Song::albumYear(song));
+                title=song.albumArtistOrComposer()+Song::constSep+Song::displayAlbum(song.albumName(), Song::albumYear(song));
             }
             track=song.trackAndTitleStr();
         }
@@ -383,7 +383,7 @@ void GroupedViewDelegate::paint(QPainter *painter, const QStyleOptionViewItem &o
         int td=index.data(Cantata::Role_AlbumDuration).toUInt();
         QString totalDuration=td>0 && td!=song.time ? Utils::formatTime(td) : QString();
         QRect duratioRect(r.x(), r.y(), r.width(), textHeight);
-        int totalDurationWidth=fm.width(totalDuration)+8;
+        int totalDurationWidth=fm.horizontalAdvance(totalDuration)+8;
         QRect textRect(r.x(), r.y(), r.width()-(rtl ? (4*constBorder) : totalDurationWidth), textHeight);
         QFont tf(f);
         tf.setBold(true);
@@ -412,11 +412,15 @@ void GroupedViewDelegate::paint(QPainter *painter, const QStyleOptionViewItem &o
     }
 
     GroupedView::drawPlayState(painter, option, r, state);
+    QVariant col = index.data(Cantata::Role_TextColor);
+    if (col.isValid()) {
+        textColor = col.value<QColor>();
+    }
     painter->setPen(textColor);
 
     int ratingsStart=rtl ? 0 : drawRatings(painter, song, r, fm, option.palette.color(QPalette::Active, QPalette::Text)); // TODO!!!
 
-    int durationWidth=showTrackDuration ? fm.width(duration)+8 : 0;
+    int durationWidth=showTrackDuration ? fm.horizontalAdvance(duration)+8 : 0;
     QRect duratioRect(r.x(), r.y(), r.width(), textHeight);
     QRect textRect(r.x(), r.y(), r.width()-durationWidth, textHeight);
     if (ratingsStart>0) {
@@ -447,7 +451,7 @@ int GroupedViewDelegate::drawRatings(QPainter *painter, const Song &song, const
         painter->setOpacity(painter->opacity()*0.75);
         const QSize &ratingSize=ratingPainter->size();
         int spacing=constBorder*2;
-        int durationWidth=fm.width("0:00:00")+spacing;
+        int durationWidth=fm.horizontalAdvance("0:00:00")+spacing;
         QRect ratingRect(r.x()+r.width()-(durationWidth+ratingSize.width()+spacing),
                          r.y()+(r.height()-ratingSize.height())/2,
                          ratingSize.width(), ratingSize.height());
@@ -552,7 +556,7 @@ void GroupedView::updateRows(qint32 row, quint16 curAlbum, bool scroll, const QM
         return;
     }
 
-    if (filterActive && model() && MPDState_Playing==MPDStatus::self()->state()) {
+    if (filterActive && MPDState_Playing==MPDStatus::self()->state()) {
         if (scroll) {
             scrollTo(model()->index(row, 0, parent), QAbstractItemView::PositionAtCenter);
         }
@@ -843,7 +847,7 @@ void GroupedView::expand(const QModelIndex &idx, bool singleOnly)
             if (!singleOnly) {
                 quint32 count=model()->rowCount(idx);
                 for (quint32 i=0; i<count; ++i) {
-                    expand(idx.child(i, 0));
+                    expand(model()->index(i, 0, idx));
                 }
             }
         } else if (AlbumHeader==getType(idx)) {
@@ -864,7 +868,7 @@ void GroupedView::collapse(const QModelIndex &idx, bool singleOnly)
             if (!singleOnly) {
                 quint32 count=model()->rowCount(idx);
                 for (quint32 i=0; i<count; ++i) {
-                    collapse(idx.child(i, 0));
+                    collapse(model()->index(i, 0, idx));
                 }
             }
         }
