@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2018 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2020 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -455,19 +455,27 @@ void NowPlayingWidget::updateInfo()
     }
 
     QString info;
-    info = MPDStatus::self()->bitrate()>0
-            ? MPDStatus::self()->samplerate()>0
-                ? info.sprintf("%d kbps, %.1f kHz", MPDStatus::self()->bitrate(), MPDStatus::self()->samplerate()/1000.0)
-                : info.sprintf("%d kbps", MPDStatus::self()->bitrate())
-            : MPDStatus::self()->samplerate()>0
-                  ? info.sprintf("%.1f kHz", MPDStatus::self()->samplerate()/1000.0)
-                  : QString();
+    if (MPDStatus::self()->bitrate()>0) {
+        info += QString().sprintf("%d kbps", MPDStatus::self()->bitrate());
+    }
+    if (MPDStatus::self()->bits()>0) {
+        if (!info.isEmpty()) {
+            info+=", ";
+        }
+        info += tr("%n bit", "", MPDStatus::self()->bits());
+    }
+    if (MPDStatus::self()->samplerate()>0) {
+        if (!info.isEmpty()) {
+            info+=", ";
+        }
+        info += QString().sprintf("%.1f kHz", MPDStatus::self()->samplerate()/1000.0);
+    }
 
     int pos=currentSongFile.lastIndexOf('.');
     if (pos>1) {
         QString ext = currentSongFile.mid(pos+1).toUpper();
 
-        if (ext.length()<5) {
+        if (ext.length()>2 && ext.length()<5) {
             if (!info.isEmpty()) {
                 info+=", ";
             }
@@ -483,7 +491,7 @@ void NowPlayingWidget::updateInfo()
 
 void NowPlayingWidget::copyInfo()
 {
-    QApplication::clipboard()->setText(track->text()+QLatin1Char(' ')+artist->text().replace(QString(" – "), QLatin1String(" ")));
+    QApplication::clipboard()->setText(track->text()+QLatin1Char(' ')+artist->text().replace(Song::constSep, QLatin1String(" ")));
 }
 
 void NowPlayingWidget::initColors()
