@@ -1,7 +1,7 @@
 /*
  * Cantata
  *
- * Copyright (c) 2011-2014 Craig Drummond <craig.p.drummond@gmail.com>
+ * Copyright (c) 2011-2020 Craig Drummond <craig.p.drummond@gmail.com>
  *
  * ----
  *
@@ -31,19 +31,27 @@ class TableView : public TreeView
     Q_OBJECT
 
 public:
-    TableView(const QString &cfgName, QWidget *parent=0, bool menuAlwaysAllowed=false);
-    virtual ~TableView() { }
+    TableView(const QString &cfgGroup, QWidget *parent=nullptr, bool menuAlwaysAllowed=false);
+    ~TableView() override { }
+    void setModel(QAbstractItemModel *m) override;
     void initHeader();
     void saveHeader();
+    void scrollTo(const QModelIndex &index, ScrollHint hint) override;
 
 private Q_SLOTS:
-    void showMenu();
+    void showMenu(const QPoint &pos);
     void toggleHeaderItem(bool visible);
     void stretchToggled(bool e);
+    void alignmentChanged();
 
 protected:
     QMenu *menu;
-    QString configName;
+    QString configGroup;
+    int menuIsForCol;
+    QAction *alignAction;
+    QAction *alignLeftAction;
+    QAction *alignCenterAction;
+    QAction *alignRightAction;
 };
 
 #endif
